//
//  ViewController.swift
//  fsevents-example
//
//  Created by Michael Ovchinnikov on 06/02/2019.
//  Copyright © 2019 Michael Ovchinnikov. All rights reserved.
//

import Cocoa

class ViewController: NSViewController {
    @IBOutlet weak var tableView: NSTableView!
    
    var fsEventsService: FSEventsService!
    var events: [FSEvent] = []

    override func viewDidLoad() {
        super.viewDidLoad()
        
        tableView.delegate = self
        tableView.dataSource = self
        
        let path = NSString(string: "~").expandingTildeInPath
        fsEventsService = FSEventsService(pathsToWatch: [path]) { [weak self] event in
            self?.events.append(event)
            self?.tableView.reloadData()
        }
        fsEventsService.start()
    }

}

extension ViewController: NSTableViewDataSource {
    func numberOfRows(in tableView: NSTableView) -> Int {
        return events.count
    }
}

extension NSUserInterfaceItemIdentifier {
    static let actionCell =  NSUserInterfaceItemIdentifier("ActionCellID")
    static let sourcePathCell =  NSUserInterfaceItemIdentifier("SourcePathCellID")
    static let destinationPathCell =  NSUserInterfaceItemIdentifier("DestinationPathCellID")
}

extension ViewController: NSTableViewDelegate {
    func tableView(_ tableView: NSTableView, viewFor tableColumn: NSTableColumn?, row: Int) -> NSView? {
        var text: String = ""
        var cellIdentifier: NSUserInterfaceItemIdentifier = .actionCell
        
        if tableColumn == tableView.tableColumns[0] {
            text = events[row].type.rawValue
            cellIdentifier = .actionCell
        } else if tableColumn == tableView.tableColumns[1] {
            text = events[row].sourcePath ?? "N/A"
            cellIdentifier = .sourcePathCell
        } else if tableColumn == tableView.tableColumns[2] {
            text = events[row].destinationPath ?? "N/A"
            cellIdentifier = .destinationPathCell
        }
        
        if let cell = tableView.makeView(withIdentifier: cellIdentifier, owner: nil) as? NSTableCellView {
            cell.textField?.stringValue = text
            return cell
        }
        return nil
    }
}
